/* ***** BEGIN LICENSE BLOCK *****
 *   Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 * 
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is Save File to.
 *
 * The Initial Developer of the Original Code is
 * Atte Kemppilä.
 * Portions created by the Initial Developer are Copyright (C) 2007
 * the Initial Developer. All Rights Reserved.
 *
 * Contributor(s):
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 * 
 * ***** END LICENSE BLOCK ***** */


const CLASS_ID = Components.ID("9e7881e3-bebe-4e10-a1c4-efb9236d9466");
const CLASS_NAME = "savefileto clean up service";
const CONTRACT_ID = "@savefileto.mozdev.org/clean-up-service;1";
const CATEGORY_ENTRY = "savefileto";

const Ci = Components.interfaces, Cc = Components.classes, Cr = Components.results;

function CleanUpService()
{
    // service has no proper interface, use wrappedJSObject
    this.wrappedJSObject = this;
}

CleanUpService.prototype =
{
    // nsISupports
    QueryInterface: function(iid)
    {
        if (iid.equals(Ci.nsIObserver) ||
            iid.equals(Ci.nsISupports) ||
            iid.equals(Ci.nsISupportsWeakReference))
            return this;
        throw Cr.NS_ERROR_NO_INTERFACE;
    },
    
    // nsIObserver
    observe: function(subject, topic, data)
    {
        switch (topic)
        {
            case "xpcom-startup":
                var obsSvc = Cc["@mozilla.org/observer-service;1"].getService(Ci.nsIObserverService);
                obsSvc.addObserver(this, "profile-after-change", true);
                obsSvc.addObserver(this, "quit-application", true);
                break;
                
            case "profile-after-change":
            case "quit-application":
                // clean up at startup and shutdown
                this.cleanUp();
                break;
                
            default:
                throw Components.Exception("unknown topic: " + topic);
        }
    },
    
    // service
    cleanUp: function(force)
    {
        var prefs = Cc["@mozilla.org/preferences-service;1"].getService(Ci.nsIPrefService);
        prefs = prefs.getBranch("extensions.savefileto.");
        if (force || prefs.getBoolPref("clearRecentFolders"))
        {
            var types = ["page", "link", "image"];
            for (var i = 0; i < types.length; i++)
                prefs.setCharPref(types[i] + ".recentFolders", "");
        }
    }
}

// nsIFactory
var CleanUpServiceFactory =
{
    instance: null,
    
    createInstance: function (outer, iid)
    {
        if (outer != null)
            throw Cr.NS_ERROR_NO_AGGREGATION;
        if (this.instance == null)
            this.instance = new CleanUpService();
        return this.instance.QueryInterface(iid);
    }
};

// nsIModule
var CleanUpServiceModule =
{
    registerSelf: function(compMgr, fileSpec, location, type)
    {
        // register
        compMgr = compMgr.QueryInterface(Ci.nsIComponentRegistrar);
        compMgr.registerFactoryLocation(CLASS_ID, CLASS_NAME, CONTRACT_ID, fileSpec, location, type);
        // add start-up observer
        var catMgr = Cc["@mozilla.org/categorymanager;1"].getService(Ci.nsICategoryManager);
        catMgr.addCategoryEntry("xpcom-startup", CATEGORY_ENTRY, CONTRACT_ID, true, true);
    },
    
    unregisterSelf: function(compMgr, fileSpec, location)
    {
        // remove start-up observer
        var catMgr = Cc["@mozilla.org/categorymanager;1"].getService(Ci.nsICategoryManager);
        catMgr.deleteCategoryEntry("xpcom-startup", CATEGORY_ENTRY, true);
        // unregister
        compMgr = compMgr.QueryInterface(Ci.nsIComponentRegistrar);
        compMgr.unregisterFactoryLocation(CLASS_ID, location);        
    },
    
    getClassObject: function(compMgr, cid, iid)
    {
        if (! cid.equals(CLASS_ID))
            throw Cr.NS_ERROR_NO_INTERFACE;
        if (! iid.equals(Ci.nsIFactory))
            throw Cr.NS_ERROR_NOT_IMPLEMENTED;
        return CleanUpServiceFactory;
    },
    
    canUnload: function(compMgr)
    {
        return true;
    }
};

function NSGetModule(compMgr, fileSpec)
{
    return CleanUpServiceModule;
}
